<?php

header('Access-Control-Allow-Origin: *');
error_reporting(E_ALL);
ini_set('display_errors', '1');
set_time_limit(0);
//echo "Getting parameters...</br>";
$params['id'] = filter_input(INPUT_POST, 'ID', FILTER_SANITIZE_STRING);
$params['x'] = filter_input(INPUT_POST, 'X', FILTER_SANITIZE_STRING);
$params['y'] = filter_input(INPUT_POST, 'Y', FILTER_SANITIZE_STRING);
$params['wood'] = filter_input(INPUT_POST, 'Wood', FILTER_SANITIZE_STRING);
$params['clay'] = filter_input(INPUT_POST, 'Clay', FILTER_SANITIZE_STRING);
$params['iron'] = filter_input(INPUT_POST, 'Iron', FILTER_SANITIZE_STRING);
$params['stone'] = filter_input(INPUT_POST, 'Stone', FILTER_SANITIZE_STRING);
$params['food'] = filter_input(INPUT_POST, 'Food', FILTER_SANITIZE_STRING);
$params['terrain'] = filter_input(INPUT_POST, 'Terrain', FILTER_SANITIZE_STRING);
$params['combat'] = filter_input(INPUT_POST, 'Combat', FILTER_SANITIZE_STRING);
$params['region'] = filter_input(INPUT_POST, 'Region', FILTER_SANITIZE_STRING);

$sourceFile = filter_input(INPUT_POST, 'sourceFile', FILTER_SANITIZE_STRING);
// Default preg_match entry for each of the fields. Used when no filter values 
// have been supplied for it.
$defaults = [];
$defaults['id'] = "/^(\-?[0-9]+\|)";
$defaults['x'] = "\-?[0-9]+\|";
$defaults['y'] = "\-?[0-9]+\|";
$defaults['wood'] = "[0-9]+\|";
$defaults['clay'] = "[0-9]+\|";
$defaults['iron'] = "[0-9]+\|";
$defaults['stone'] = "[0-9]+\|";
$defaults['food'] = "[0-9]+\|";
$defaults['terrain'] = "[0-9]+\|";
$defaults['combat'] = "[0-9]+\|";
$defaults['region'] = "[0-9]+";

// Check the parameters
$paramCheck = checkParams($params);
foreach ($paramCheck as $key => $value) {
    if ($value === FALSE) {
        echo "Error in " . $key . " parameter => " . $params[$key] . "</br>";
    }
}
// var_dump($params);
// Build the pattern to match against
$regexParts = [];
foreach ($params as $key => $value) {
    $filterArray[$key] = parseParam($value);
//    echo $key . " length: " . count($filterArray[$key]) . "==>";
    if (count($filterArray[$key]) !== 0) {
        $regexParts[$key] = "(" . implode($filterArray[$key], "|") . ")\|";
    } else {
        $regexParts[$key] = $defaults[$key];
    }
//    echo $regexParts[$key]."</br>";
}
$pattern = implode($regexParts, "");
if (substr($pattern, -1) === "|") {
    $pattern = rtrim($pattern, "\|");
}
$pattern = $pattern . "/m";
// echo $pattern;
// Open the input and output files
$inHandle = @fopen("data/datafile_worldmap.txt", "r");
//$inHandle = @fopen($sourceFile, "r");
$outHandle = fopen("data/results.txt", "w");

// Scan the input file
$numMatches = scanFile($inHandle, $outHandle, $pattern);
//echo "File scanned...</br>";
// Read the results file and send the contents back to the server as html
$resultsFile = fopen("data/results.txt", "r");
echo "Total number of matches: " . $numMatches . "</br>";
if ($numMatches < 500) {
    while (!feof($resultsFile)) {
        $line = fgets($resultsFile, 4096);
        $parts = explode("|", $line);
        if (count($parts) === 11) {
            $thisline = '<a href="https://elgea.illyriad.co.uk/#/World/Map/' . $parts[1] . '/' . $parts[2] . '" target="_blank">' . $parts[1] . '|' . $parts[2] . '</a>';
            echo $thisline . ", ";
        }
    }
}
fclose($resultsFile);

// ============================================================================
// Check the input file for matching lines and store the results in the output array.
// Function to find matching lines in the input file
function scanFile($fh, $outHandle, $pattern) {
    $matched = 0;
//    echo $pattern . "</br>";
    while (!feof($fh)) {
        $line = fgets($fh, 4096);
        if (preg_match($pattern, $line)) {
//            echo $line . "</br>";
            fwrite($outHandle, $line);
            $matched += 1;
        }
    }
    fclose($fh);
    fclose($outHandle);
    return $matched;
}

// Check that the parameters supplied are valid
function checkParams($params) {
//    echo "Checking parameters...</br>";
    $checkResults = [];
    $flag = true;
    foreach ($params as $key => $value) {
        if ((preg_match_all('/[-0123456789,\.]/', $value) OR $value === '')) {
            $checkResults[$key] = true;
        } else {
            $checkResults[$key] = false;
            $flag = false;
        }
    }
    if ($flag === false) {
        foreach ($checkResults as $key => $value) {
            if ($value === FALSE) {
                echo "Error in " . $key . " parameter => " . $params[$key] . "</br>";
            }
        }
        exit();
    }
    return $checkResults;
}

// Function to parse a parameter into an array of acceptable values
function parseParam($param) {
//    echo "Parsing params...</br>";
    $accept = [];
    if ($param !== '') {
// Split on any commas
        $parts = explode(",", $param);
// Check each of the parts to see if they are ranges
        foreach ($parts as $value) {
            $parts2 = explode("...", $value);
            if (count($parts2) > 1) {
// This is a range. Check that the parts are in order lower to higher
                sort($parts2, SORT_NUMERIC);
                for ($i = $parts2[0]; $i <= $parts2[1]; $i++) {
                    array_push($accept, intval($i));
                }
            } else {
// This is a single value
                array_push($accept, intval($value));
            }
        }
    }
    return $accept;
}
